const char* window_title="LD20 by DrPetter";

#include "Vector.h"
#include "syscode.h"

#define rnd(n) (rand()%(n+1))
#define frnd() (((float)(rand()&0xFFF))/0xFFF)
#define PI 3.14159265f

Vector Normalize(Vector v)
{
	v.NormalizeSafe();
	return v;
}

void glVertexV(Vector& vec)
{
	glVertex3f(vec.x, vec.y, vec.z);
}

void glColorV(Vector& vec)
{
	glColor3f(vec.x, vec.y, vec.z);
}

float spow(float x, float y)
{
	if(x<0.0f)
		return -pow(-x, y);
	else
		return pow(x, y);
}


int stage=1;
bool cheat=false;


int slide_t=0;
int slide_step=0;
bool slide_active=true;


struct Pork
{
	Vector pos;
	float speed;
	
	float droll;
	float dtilt;
	float dpan;

	Vector forward;
	Vector right;
	Vector up;
};

Pork ship;
Pork camera;

float cam_follow;
float vroll;
float danger;
float safe;
int engines;

bool eclipse;
bool cityscape;

const int debrisc=6000;
Vector debris[debrisc];
float debrisf[debrisc];

const int trailc=8000;
Vector trail[trailc];
float trailf[trailc];

const int wurmc=900;
Vector wurm[wurmc];
float wurmf[wurmc];
int wurmic;
float wurmf0;

int wallt;
float walla;

struct DParticle
{
	Vector pos;
	Vector vel;
	int t;
};
const int splodec=500;
DParticle splode[splodec];

struct Impact
{
	int t;
	Vector pos;
};

const int impc=12;
const int impl=1400;
Impact impacts[impc];

int winning;

bool fade=true;
float fade_a=1.0f;
float fade_d=-1.0f/60;


void Orthon(Pork& p)
{
	p.right=p.forward%p.up;
	p.up=p.right%p.forward;
	p.forward.NormalizeSafe();
	p.right.NormalizeSafe();
	p.up.NormalizeSafe();
}

void RollAxes(Vector& forward, Vector& right, Vector& up, float angle)
{
	Vector right1=right*cos(angle)+up*sin(angle);
	Vector up1=-right*sin(angle)+up*cos(angle);
	right=right1;
	up=up1;
}

void TiltAxes(Vector& forward, Vector& right, Vector& up, float angle)
{
	RollAxes(right, forward, up, angle);
}

void PanAxes(Vector& forward, Vector& right, Vector& up, float angle)
{
	RollAxes(up, right, forward, angle);
}

Vector Rotate(Vector v, Vector axis, float angle)
{
	Vector af=axis;
	Vector au=axis%v;
	Vector ar=af%au;
	au=ar%af;
	af.NormalizeSafe();
	ar.NormalizeSafe();
	au.NormalizeSafe();
	float fx=v*af;
	float fy=v*ar;
	RollAxes(af, ar, au, angle);
	return af*fx+ar*fy;
}

void RunPork(Pork& p)
{
	p.pos+=p.forward*p.speed;
	
	RollAxes(p.forward, p.right, p.up, p.droll);
	TiltAxes(p.forward, p.right, p.up, p.dtilt);
	PanAxes( p.forward, p.right, p.up, p.dpan);
}

bool InitGame()
{
	ship.pos=Vector(0,0,8);
	ship.forward=Vector(0,0,-1);
	ship.right=Vector(1,0,0);
	ship.up=Vector(0,1,0);
	ship.droll=0.0f;
	ship.dtilt=0.0f;
	ship.dpan=0.0f;
	ship.speed=0.02f;
	vroll=0.0f;
	
	danger=0;
	safe=0;
	engines=2;
	
	camera.pos=Vector(3,0,7);
	camera.forward=Vector(0,0,-1);
	camera.right=Vector(1,0,0);
	camera.up=Vector(0,1,0);
	PanAxes(camera.forward, camera.right, camera.up, -1.7f);
	RollAxes(camera.forward, camera.right, camera.up, 0.1f);
	camera.droll=0.0f;
	camera.dtilt=0.0f;
	camera.dpan=0.0f;
	camera.speed=0.0f;
	
	cam_follow=0.0f;
	
	for(int i=0;i<debrisc;i++)
	{
		debris[i]=Vector(frnd()-0.5f, frnd()-0.5f, frnd()-0.5f);
		debris[i].NormalizeSafe();
//		debris[i]=Vector(1,0,0);
		debris[i]=Rotate(debris[i], Vector(0,1,0), frnd()*PI);
		debris[i]=Rotate(debris[i], Vector(0,0,1), frnd()*PI);
		float powf=0.25f;
		if(stage==2)
			powf=0.02f;
		debris[i]*=pow(frnd(), powf)*3.0f;
//		debris[i]=Vector(frnd()-0.5f, frnd()-0.5f, frnd()-0.5f);
//		debris[i]*=20.0f;
		debrisf[i]=0.6f+frnd()*0.4f;
	}
	
	eclipse=(stage==3);
	cityscape=(stage!=4);

	for(int i=0;i<trailc;i++)
	{
		trail[i]=Vector(0,1000,0);
		trailf[i]=frnd()*PI*2;
	}
	
	for(int i=0;i<impc;i++)
		impacts[i].t=0;
	
	for(int i=0;i<splodec;i++)
		splode[i].t=0;

	for(int i=0;i<wurmc;i++)
		wurm[i]=Vector(0,1000,0);
	wurmic=0;
	wurmf0=0.0f;
	
	wallt=0;
	walla=0.0f;

	winning=0;

	return true;
}

void ExitGame()
{
}

void DrawBar(float x, float y, float w, float h)
{
	glVertex3f(x, y, 0);
	glVertex3f(x, y+h, 0);
	glVertex3f(x+w, y+h, 0);
	glVertex3f(x+w, y, 0);
}

void DrawBarT(float x, float y, float w, float h)
{
	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex3f(x, y, 0);
	glTexCoord2f(0,1);
	glVertex3f(x, y+h, 0);
	glTexCoord2f(1,1);
	glVertex3f(x+w, y+h, 0);
	glTexCoord2f(1,0);
	glVertex3f(x+w, y, 0);
	glEnd();
}

void DrawSlides()
{
//	int slide_t=0;
//	int slide_step=0;
//	bool slide_active=true;

	slide_t++;
	float ft0=(float)slide_t*0.001f;
	if(ft0>1.0f)
		ft0=1.0f;
	float ft=1.0f-ft0;
	ft*=ft;
	ft=1.0f-ft;

	static int slide_initstep=-1;
	static bool skipping=false;
	if(!fade && slide_t<300)
	{
		if(KeyHit(VK_SPACE) || KeyHit(VK_RETURN))
		{
//			slide_t=300;
			fade=true;
			fade_d=1.0f/60;
			skipping=true;
		}
	}
	if(slide_t==300 && !fade)
	{
		fade=true;
		fade_d=1.0f/60;
	}
	if(slide_t==360 || (skipping && fade_a==1.0f))
	{
		slide_t=0;
		slide_step++;
		fade=true;
		fade_d=-1.0f/60;
		skipping=false;
	}

	glEnable(GL_TEXTURE_2D);
	glColor3f(1,1,1);


	if(slide_step==0)
	{
		if(slide_initstep!=slide_step)
		{
			LoadTexture(1, "data\\intro1.png");
			LoadTexture(0, "data\\intro-txt1.png");
			slide_initstep=slide_step;
		}
		UseTexture(1);
		DrawBarT(-0.5f,0.2f-ft*0.2f,1,1);
		UseTexture(0);
		DrawBarT(-0.5f,0.7f,1,0.5f);
	}

	if(slide_step==1)
	{
		if(slide_initstep!=slide_step)
		{
			LoadTexture(1, "data\\intro2.png");
			LoadTexture(0, "data\\intro-txt2.png");
			slide_initstep=slide_step;
		}
		UseTexture(1);
		DrawBarT(-0.5f,-0.3f+ft*0.3f,1,1);
		UseTexture(0);
		DrawBarT(-0.5f,0.7f,1,0.5f);
	}

	if(slide_step==2)
	{
		if(slide_initstep!=slide_step)
		{
			LoadTexture(1, "data\\intro3.png");
			LoadTexture(0, "data\\intro-txt3.png");
			slide_initstep=slide_step;
		}
		UseTexture(1);
		DrawBarT(-0.3f-ft*0.4f,0.0f,1,1);
		UseTexture(0);
		DrawBarT(-0.5f,0.7f,1,0.5f);
	}

	if(slide_step==3)
	{
		if(slide_initstep!=slide_step)
		{
//			LoadTexture(1, "data\\intro4.png");
			LoadTexture(0, "data\\intro-txt4.png");
			slide_initstep=slide_step;
		}
//		UseTexture(1);
//		DrawBarT(-0.3f-ft*0.4f,0.0f,1,1);
		UseTexture(0);
		DrawBarT(-0.5f,0.7f,1,0.5f);
	}

	if(slide_step==4)
	{
		if(slide_initstep!=slide_step)
		{
//			LoadTexture(1, "data\\intro5.png");
			LoadTexture(0, "data\\intro-txt5.png");
			slide_initstep=slide_step;
		}
//		UseTexture(1);
//		DrawBarT(-0.3f-ft*0.4f,0.0f,1,1);
		UseTexture(0);
		DrawBarT(-0.5f,0.7f,1,0.5f);
	}

	if(slide_step==5)
	{
		if(slide_initstep!=slide_step)
		{
//			LoadTexture(1, "data\\intro6.png");
			LoadTexture(0, "data\\intro-txt6.png");
			slide_initstep=slide_step;
		}
//		UseTexture(1);
//		DrawBarT(-0.3f-ft*0.4f,0.0f,1,1);
		UseTexture(0);
		DrawBarT(-0.5f,0.7f,1,0.5f);
	}

	if(slide_step==6)
	{
		if(slide_initstep!=slide_step)
		{
			LoadTexture(1, "data\\this.png");
			LoadTexture(0, "data\\intro-txt7.png");
			slide_initstep=slide_step;
		}
		UseTexture(1);
		DrawBarT(-0.5f,0.2f-ft*0.2f,1,1);
		UseTexture(0);
		DrawBarT(-0.5f,0.7f,1,0.5f);
	}

	if(slide_step==7)
	{
		slide_active=false;
	}

	if(slide_step==8)
	{
		if(slide_initstep!=slide_step)
		{
//			LoadTexture(1, "data\\level.png");
			LoadTexture(0, "data\\level2-txt.png");
			slide_initstep=slide_step;
		}
//		UseTexture(1);
//		DrawBarT(-0.3f-ft*0.4f,0.0f,1,1);
		UseTexture(0);
		DrawBarT(-0.5f,0.7f,1,0.5f);
	}

	if(slide_step==9)
	{
		slide_active=false;
	}

	if(slide_step==10)
	{
		if(slide_initstep!=slide_step)
		{
//			LoadTexture(1, "data\\level.png");
			LoadTexture(0, "data\\level3-txt.png");
			slide_initstep=slide_step;
		}
//		UseTexture(1);
//		DrawBarT(-0.3f-ft*0.4f,0.0f,1,1);
		UseTexture(0);
		DrawBarT(-0.5f,0.7f,1,0.5f);
	}

	if(slide_step==11)
	{
		slide_active=false;
	}

	if(slide_step==12)
	{
		if(slide_initstep!=slide_step)
		{
//			LoadTexture(1, "data\\level.png");
			LoadTexture(0, "data\\level4-txt.png");
			slide_initstep=slide_step;
		}
//		UseTexture(1);
//		DrawBarT(-0.3f-ft*0.4f,0.0f,1,1);
		UseTexture(0);
		DrawBarT(-0.5f,0.7f,1,0.5f);
	}

	if(slide_step==13)
	{
		slide_active=false;
	}

	if(slide_step==14)
	{
		if(slide_initstep!=slide_step)
		{
//			LoadTexture(1, "data\\level.png");
			LoadTexture(0, "data\\win-txt.png");
			slide_initstep=slide_step;
		}
//		UseTexture(1);
//		DrawBarT(-0.3f-ft*0.4f,0.0f,1,1);
		UseTexture(0);
		DrawBarT(-0.5f,0.7f,1,0.5f);
	}

	if(slide_step==15)
	{
		exit(0);
	}

	glDisable(GL_TEXTURE_2D);
}

void DrawFade()
{
	if(fade)
	{
		fade_a+=fade_d;
		if(fade_a<0.0f)
		{
			fade_a=0.0f;
			fade=false;
		}
		if(fade_a>1.0f)
			fade_a=1.0f;
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		if(winning>0)
		{
			float fa=fade_a*1.2f-0.2f;
			if(fa<0.0f)
				glColor4f(1,1,0.8f,-fa*3.0f);
			else
				glColor4f(0,0,0,fa);
		}
		else
			glColor4f(0,0,0,fade_a);
		glBegin(GL_QUADS);
		DrawBar(-2,0,4,1);
		glEnd();
		glDisable(GL_BLEND);
	}
}

void DrawGame()
{
	SetProjection(true);
	
	glClear(GL_COLOR_BUFFER_BIT);

	glDisable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);


	if(slide_active)
	{
		SetProjection(false);
		DrawSlides();
		DrawFade();
		return;
	}


//	glTranslatef(0,-1.0f,-6);
	glRotatef(10, 1,0,0);


	float cmat[16];
	int ci=0;

	cmat[ci++]=camera.right.x; cmat[ci++]=camera.up.x; cmat[ci++]=-camera.forward.x; cmat[ci++]=0.0f;
	cmat[ci++]=camera.right.y; cmat[ci++]=camera.up.y; cmat[ci++]=-camera.forward.y; cmat[ci++]=0.0f;
	cmat[ci++]=camera.right.z; cmat[ci++]=camera.up.z; cmat[ci++]=-camera.forward.z; cmat[ci++]=0.0f;
	cmat[ci++]=0.0f; cmat[ci++]=0.0f; cmat[ci++]=0.0f; cmat[ci++]=1.0f;

//	cmat[ci++]=camera.right.x;   cmat[ci++]=camera.right.y;   cmat[ci++]=camera.right.z;   cmat[ci++]=0.0f;
//	cmat[ci++]=camera.up.x;      cmat[ci++]=camera.up.y;      cmat[ci++]=camera.up.z;      cmat[ci++]=0.0f;
//	cmat[ci++]=camera.forward.x; cmat[ci++]=camera.forward.y; cmat[ci++]=camera.forward.z; cmat[ci++]=0.0f;
//	cmat[ci++]=0.0f; cmat[ci++]=0.0f; cmat[ci++]=0.0f; cmat[ci++]=1.0f;

	glMultMatrixf(cmat);

	Vector camp=camera.pos;
	camp+=camera.up*1;
	camp-=camera.forward*6;
//	glTranslatef(-camera.pos.x, -camera.pos.y, -camera.pos.z);
	glTranslatef(-camp.x, -camp.y, -camp.z);



/*	glLineWidth(1);
	glBegin(GL_LINES);
	glColor3f(0,0,0);
	glVertexV(ship.pos);
	glColor3f(0.5f,0.5f,0.5f);
	glVertexV(Normalize(ship.pos)*3);
	glEnd();
*/


	glColor3f(1,1,1);
	glBegin(GL_TRIANGLES);
	Vector spr=Vector(0,0,0);
	for(int i=0;i<=90;i++)
	{
		float a=(float)i/90*2*PI;
		float x=cos(a);
		float y=sin(a);
		Vector r=camera.right*x+camera.up*y;
		r*=3.0f;
		Vector scc=camp+Vector(0,40,0);
		r+=scc;
		if(i>0)
		{
			glVertexV(scc);
			glVertexV(spr);
			glVertexV(r);
		}
		spr=r;
	}
	if(eclipse)
	{
		glColor3f(0.2f,0,0.1f);
		for(int i=0;i<=90;i++)
		{
			float a=(float)i/90*2*PI;
			float x=cos(a)*0.9f+0.1f;
			float y=sin(a);
			Vector r=camera.right*x+camera.up*y;
			r*=2.9f;
			Vector scc=camp+Vector(0,38,0);
			r+=scc;
			if(i>0)
			{
				glVertexV(scc);
				glVertexV(spr);
				glVertexV(r);
			}
			spr=r;
		}
	}
	glEnd();

	if(cityscape)
	{
		glPointSize(2);
		if(stage==3)
			glBegin(GL_POINTS);
		else
			glBegin(GL_LINES);
		for(int j=0;j<2;j++)
			for(int cy=-40;cy<=40;cy++)
			{
				Vector pr1=Vector(0,0,0);
				Vector pr2=Vector(0,0,0);
				float pc1=0.0f;
				float pc2=0.0f;
				float yyy=(float)cy/2+sin((float)cy/2*1.37f)*0.4f;
				yyy+=cos((float)cy*1.69f)*0.3f;
				for(int cx=-10;cx<=10;cx++)
				{
					Vector r=Vector((float)cx*2, 0, yyy);
					r.z=spow(r.z/20, 1.7f)*20;
					if(j==1)
					{
						float t=r.x;
						r.x=r.z;
						r.z=t;
					}
					float rc=1.0f-r.Length()/25;
					if(rc<0.0f)
						rc=0.0f;
					rc*=rc*rc*1;
					if(rc>0.2f)
						rc=0.2f;
					if(stage==3)
						rc*=3;
					r+=camp;
					r.y-=3;
					if(cx>-10)
					{
						glColorV(Vector(0.4f,0.45f,0.45f)*pc1);
						glVertexV(pr1);
						glColorV(Vector(0.4f,0.45f,0.45f)*rc);
						glVertexV(r);
					}
					pr1=r;
					pc1=rc;
				}
			}
		glEnd();
	}

	glColor3f(0.4f,0.4f,0.4f);
	glBegin(GL_LINES);
	for(int i=0;i<=120;i++)
	{
		float a=(float)i/120*2*PI;
		float x=cos(a);
		float z=sin(a);
		Vector r=Vector(x, 0, z)*5;
		r+=camp;
		if(i>0)
			glVertexV(r);
		if(i<120)
			glVertexV(r);
	}
	glEnd();


	glPointSize(1.0f);
	glBegin(GL_POINTS);
	for(int i=0;i<debrisc;i++)
	{
		if(debrisf[i]<0.1f)
			continue;
		glColorV(Vector(0.4f,0.7f,1.0f)*(3.0f-debrisf[i]*2.3f));
		glVertexV(debris[i]);
	}
	for(int i=0;i<trailc;i++)
	{
		if(trail[i].y>100)
			continue;
		float f=(sin(trailf[i])+1.0f)*0.5f;
		f*=f*f*f*3.5f;
		glColorV(Vector(1.0f,0.2f,0.25f)*f);
		trailf[i]+=0.04f;
		if(trailf[i]>PI*2)
			trailf[i]-=PI*2;
		glVertexV(trail[i]);
	}
	glEnd();

	if(stage==3)
	{
		glPointSize(4.0f);
		glBegin(GL_POINTS);
		glColor3f(0.8f,0,0);
		for(int i=0;i<wurmc;i++)
		{
			if(wurm[i].y>100)
				continue;
			glVertexV(wurm[i]+Vector(cos(wurmf[i]), sin(wurmf[i]), cos(wurmf[i]*0.7f))*0.04f);
		}
		glEnd();
		glPointSize(2.0f);
		glBegin(GL_POINTS);
		glColor3f(0.8f,1,0.6f);
		for(int i=0;i<wurmc;i++)
		{
			if(wurm[i].y>100)
				continue;
			glVertexV(wurm[i]+Vector(cos(wurmf[i]), sin(wurmf[i]), cos(wurmf[i]*0.7f))*0.04f);
		}
		glEnd();
	}

/*
	int sivx=(int)ship.pos.x;
	int sivy=(int)ship.pos.y;
	int sivz=(int)ship.pos.z;
	glPointSize(2.0f);
	glBegin(GL_POINTS);
	for(int x=-6;x<=6;x++)
		for(int y=-6;y<=6;y++)
			for(int z=-6;z<=6;z++)
			{
				int nm=0;
				if(abs(x)!=6)
					nm++;
				if(abs(y)!=6)
					nm++;
				if(abs(z)!=6)
					nm++;
				if(nm>2)
					continue;
				if(nm==2)
					glColor3f(0.6f,0.4f,0.3f);
				else
					glColor3f(1.0f,0.4f,0.0f);
//				if(x==sivx && y==sivy && z==sivz)
//				else
//					glColor3f(0.4f,0.1f,0.2f);

				Vector np=Vector((float)x,(float)y,(float)z);
				np.NormalizeSafe();
				np*=3.0f;

				glVertexV(np);
			}
	glEnd();
*/

//	Vector cv=camera.pos-camera.forward*6+camera.up*1;
//	float cdx=(cv-camera.forward*(cv*camera.forward)).Length();
//	Vector cdxv=cv-camera.forward*(cv*camera.forward);
	float cdy=fabs(camp*camera.forward);
//	cdx/=3.0f;
//	cdxv*=1.0f/3;
	cdy/=3.0f;
	glColor3f(1,1,1);
	Vector pr=Vector(0,0,0);
	glBegin(GL_LINES);
	for(int i=0;i<=120;i++)
	{
		float a=(float)i/120*2*PI;
		float x=cos(a);
		float y=sin(a);
		
		Vector r=camera.right*x+camera.up*y;

//		float cdrx=(r-camera.forward*(r*camera.forward)).Length();
//		float cdx1=cdx-cdrx;
		float cdx=camp*Normalize(r)/3;
				
		float ia=0.0f;
		float dia=PI*0.49f;
		for(int t=0;t<16;t++)
		{
			float s1=sin(ia)/cos(ia);
			float s2=(cos(ia)-cdx)/(cdy-sin(ia));
			if(s1<s2)
				ia+=dia;
			else
				ia-=dia;
			dia*=0.5f;
		}
		r*=3;
		r*=cos(ia);
		r-=camera.forward*sin(ia)*3;

		float rd=(r-camp)*camera.forward;
		if(rd>0.1f)
		{
			float c=r.y/r.Length();
			Vector l0=Vector(0.92f,0.96f,1);
			Vector l1=Vector(0.52f,0.47f,0.55f);
			glColorV(l1+(l0-l1)*c);
			if(i>0)
			{
				glVertexV(pr);
				glVertexV(r);
			}
		}
		pr=r;
	}
	glEnd();



	for(int i=0;i<impc;i++)
	{
		if(impacts[i].t<=0)
			continue;
		Vector normal=Normalize(impacts[i].pos);
		Vector i_forward=normal;
		Vector i_right=normal%Vector(0,1,0);
		i_right.NormalizeSafe();
		Vector i_up=i_forward%i_right;
		float ir=pow((float)impacts[i].t/impl, 0.4f)*0.9f;

		glLineWidth(1);
		glColorV(Vector(1,1.2f,2.8f)*(2.0f-(float)impacts[i].t/impl*2)*0.7f);
		Vector ipp=impacts[i].pos-normal*(1.0f-cos(asin(ir/3)))*3;
		Vector pr=Vector(0,0,0);
		glBegin(GL_LINES);
		for(int j=0;j<=80;j++)
		{
			float a=(float)j/80*2*PI;
			float x=cos(a)*ir;
			float y=sin(a)*ir;

			Vector r=ipp+i_right*x+i_up*y;
			if((r-camp)*camera.forward>0)
			{
				if(j>0)
				{
					glVertexV(pr);
					glVertexV(r);
				}
			}
			pr=r;
/*			if(j>0)
				glVertexV(ipp+i_right*x+i_up*y);
			if(j<80)
				glVertexV(ipp+i_right*x+i_up*y);*/
		}
		glEnd();
	}

	if(stage==4)
	{
		Vector walln=Vector(sin(walla), 0, -cos(walla));
		Vector wallr=Vector(cos(walla), 0, sin(walla));
		Vector wallu=wallr%walln;
		glColor3f(0.7f,1,0.2f);
		for(int i=0;i<7;i++)
		{
			float ra=3.0f-(float)i*0.3f;
			Vector pr=Vector(0,0,0);
			glBegin(GL_LINES);
			for(int j=0;j<=80;j++)
			{
				float a=(float)j/80*2*PI;
				float x=cos(a)*ra;
				float y=sin(a)*ra;
	
				Vector r=wallr*x+wallu*y;
				if(j>0)
				{
					glVertexV(pr);
					glVertexV(r);
				}
				pr=r;
			}
			glEnd();
		}
	}


	if(danger>20)
	{
		static float dt=0.0f;
		float dff=(float)(danger-20)/300;
		dt+=0.44f+dff*1.4f;
		float danr=(cos(dt)+1)*dff*0.6f+dff*0.5f;
		glColor3f(0.7f,0.1f,0);
		glBegin(GL_TRIANGLES);
		Vector spr=Vector(0,0,0);
		for(int i=0;i<=50;i++)
		{
			float a=(float)i/50*2*PI;
			float x=cos(a);
			float y=sin(a);
			Vector r=camera.right*x+camera.up*y;
			r*=danr;
			Vector scc=ship.pos;
			r+=scc;
			if(i>0)
			{
				glVertexV(scc);
				glVertexV(spr);
				glVertexV(r);
			}
			spr=r;
		}
		glEnd();
	}


	glLineWidth(2);
	glBegin(GL_LINES);
	for(int j=0;j<splodec;j++)
		if(splode[j].t>0)
		{
			float tf=(float)splode[j].t/100;
			tf*=tf;
			glColorV(Vector(1,0.3f,0.2f)*tf);
			glVertexV(splode[j].pos+Vector(frnd()-0.5f,frnd()-0.5f,frnd()-0.5f)*0.06f);
			glVertexV(splode[j].pos+splode[j].vel*3+Vector(frnd()-0.5f,frnd()-0.5f,frnd()-0.5f)*0.06f);
		}
	glEnd();
	glLineWidth(1);



	if(engines>0)
	{
		glPointSize(3.0f);
		glColor3f(0,0.7f,0.7f);
		glBegin(GL_POINTS);
		glVertexV(ship.pos-ship.forward*0.02f);
		glEnd();
		
		glColor3f(1,1,1);
		float sscale=0.3f;
		Vector shb_r=ship.right;
		Vector shb_u=ship.up;
		RollAxes(ship.forward, ship.right, ship.up, spow(vroll*35.0f, 0.7f));
		glBegin(GL_LINES);
		Vector sp=ship.pos-ship.forward*sscale*0.5f;
		glVertexV(sp+ship.forward*sscale*0.3f); glVertexV(sp+ship.forward*sscale*0.0f+ship.right*sscale*0.4f);
		glVertexV(sp+ship.forward*sscale*0.3f); glVertexV(sp+ship.forward*sscale*0.0f-ship.right*sscale*0.4f);
		glVertexV(sp); glVertexV(sp-ship.forward*sscale*0.2f+ship.up*sscale*0.2f+ship.right*sscale*0.2f);
		glVertexV(sp); glVertexV(sp-ship.forward*sscale*0.2f+ship.up*sscale*0.2f-ship.right*sscale*0.2f);
		glVertexV(sp); glVertexV(sp-ship.forward*sscale*0.1f-ship.up*sscale*0.1f+ship.right*sscale*0.3f);
		glVertexV(sp); glVertexV(sp-ship.forward*sscale*0.1f-ship.up*sscale*0.1f-ship.right*sscale*0.3f);
		glEnd();
	
		glLineWidth(2);
		glBegin(GL_LINES);
		glColor3f(1,1,1);
		glVertexV(sp); glVertexV(sp+ship.forward*sscale*0.5f);
		glColor3f(0.9f,0.2f,0.1f);
		if(engines>0)
		{
			glVertexV(sp+ship.forward*sscale*0.15f+ship.right*sscale*0.4f*0.5f+ship.forward*sscale*0.15f);
			glVertexV(sp+ship.forward*sscale*0.15f+ship.right*sscale*0.4f*0.5f-ship.forward*sscale*0.15f);
		}
		if(engines>1)
		{
			glVertexV(sp+ship.forward*sscale*0.15f-ship.right*sscale*0.4f*0.5f+ship.forward*sscale*0.15f);
			glVertexV(sp+ship.forward*sscale*0.15f-ship.right*sscale*0.4f*0.5f-ship.forward*sscale*0.15f);
		}
		glEnd();
		glLineWidth(1);
	
		ship.right=shb_r;
		ship.up=shb_u;
	}



	SetProjection(false);
	
	int debdone=0;
	for(int i=0;i<debrisc;i++)
		if(debrisf[i]<0.1f)
			debdone++;
	int debdone_steps=debdone*31/debrisc/3;
	if(debdone_steps>10) debdone_steps=10;

	glBegin(GL_QUADS);
	for(int i=0;i<10;i++)
	{
		if(i<debdone_steps)
			glColor3f(0.5f,0.7f,1.0f);
		else
			glColor3f(0.45f,0.4f,0.4f);
		DrawBar(-0.1f+(float)i*0.02f, 0.07f, 0.015f, 0.045f);
	}
	glEnd();
	
	int wingoal=10;
	if(cheat)
		wingoal=1;
	if(debdone_steps==wingoal && winning==0)
	{
		winning=1;
		fade=true;
		fade_a=0.0f;
		fade_d=1.0f/200;
	}
	
	DrawFade();
}


bool GameStep()
{
	static DWORD timestamp=0;
	while(timeGetTime()<timestamp+16)
		Sleep(0);
	timestamp=timeGetTime();

	if(KeyHit(VK_ESCAPE))
		return false;
	
	if(slide_active)
		return true;

	bool playing=true;
	if(engines==0)
		playing=false;
	if(winning>0)
		playing=false;

	static bool pinside=false;
	static int over_delay=0;
	if(!playing && engines<=0)
	{
		over_delay++;
		if(over_delay==180-60)
		{
			fade=true;
			fade_a=0.0f;
			fade_d=1.0f/60;
		}
		if(over_delay>180)
		{
			pinside=false;
			over_delay=0;
			InitGame();
			fade=true;
			fade_a=1.0f;
			fade_d=-1.0f/60;
			return true;
		}
	}
	
	if(!playing && winning>0)
	{
		danger=0;
		winning++;
		if(winning>200)
		{
			stage++;
			slide_step++;
			slide_active=true;
			
			pinside=false;
			over_delay=0;
			InitGame();
			fade=true;
			fade_a=1.0f;
			fade_d=-1.0f/60;
			return true;
		}
	}

	if(playing)
	{
		float tspd=0.0008f;
		if(KeyPressed(VK_RIGHT))
			ship.dpan-=tspd;
		if(KeyPressed(VK_LEFT))
			ship.dpan+=tspd;
		if(KeyPressed(VK_UP))
			ship.dtilt-=tspd;
		if(KeyPressed(VK_DOWN))
			ship.dtilt+=tspd;
	
		float ddc=0.98f;
		ship.droll*=ddc;
		ship.dtilt*=ddc;
		ship.dpan*=ddc;
		RunPork(ship);
		Orthon(ship);
		
		vroll+=(ship.dpan-vroll)*0.03f;
	
		static int trailic=0;
		if(ship.pos.Length()<3.0f)
		{
			trailic++;
			trailic%=trailc;
			trail[trailic]=ship.pos-ship.forward*0.07f+Vector(frnd()-0.5f, frnd()-0.5f, frnd()-0.5f)*0.05f;
		}
	}

	bool inside=false;
	if(ship.pos.Length()<3.0f)
		inside=true;

	if(inside!=pinside)
	{
		for(int i=0;i<impc;i++)
			if(impacts[i].t<=0)
			{
				impacts[i].t=1;
				impacts[i].pos=Normalize(ship.pos)*3;
				break;
			}
	}

	for(int i=0;i<impc;i++)
	{
		if(impacts[i].t<=0)
			continue;
		impacts[i].t++;
		if(impacts[i].t>=impl)
			impacts[i].t=0;
	}

	if(stage==4)
	{
		wallt++;
		walla=(float)wallt*0.0032f;
	}

	int collecting=0;
	if(playing)
	{
		for(int i=0;i<debrisc;i++)
		{
			if(debrisf[i]<0.1f)
				continue;
			debrisf[i]+=(1.0f-debrisf[i])*0.02f;
			Vector r=debris[i]-ship.pos;
			float d=r.Length();
			if(d>1.0f || !inside)
				continue;
			
			collecting++;
			
			d=1.0f-d;
			d*=d;
			debris[i]+=(ship.pos-debris[i])*d*0.3f;
			debrisf[i]+=-debrisf[i]*d*0.3f;
		}
		float dangerplus=0.0f;
		for(int i=0;i<trailc;i++)
		{
			Vector r=trail[i]-ship.pos;
			if(r*ship.forward<0)
				continue;
			float d=r.Length()/0.8f;
			if(d>1.0f || !inside)
				continue;
			d=1.0f-d;
			dangerplus+=d*0.2f;
		}
		if(stage==3)
		{
			for(int i=0;i<wurmc;i++)
			{
				Vector r=wurm[i]-ship.pos;
				if(r*ship.forward<0)
					continue;
				float d=r.Length()/0.8f;
				if(d>1.0f || !inside)
					continue;
				d=1.0f-d;
				dangerplus+=d*0.4f;
				if(d>0.5f)
					danger+=d*0.15f;
			}
		}
		if(stage==4)
		{
			Vector walln=Vector(sin(walla), 0, -cos(walla));
			float sd=ship.pos*walln;
			if(fabs(sd)<0.15f && ship.pos.Length()>1.0f)
				danger+=90.0f;
		}

		if(inside)
			dangerplus+=1.0f;
		else
			danger=0;
		dangerplus-=(float)collecting/18;
		if(dangerplus>1.5f)
			dangerplus=1.5f;
		danger+=dangerplus;
		if(danger<0)
			danger=0;
		if(danger>120)
		{
			// boom
			for(int i=0;i<200;i++)
				for(int j=0;j<splodec;j++)
					if(splode[j].t==0)
					{
						splode[j].t=100+rnd(100);
						splode[j].pos=ship.pos;
						splode[j].vel=Vector(frnd()-0.5f, frnd()-0.5f, frnd()-0.5f)*0.3f;
						break;
					}
			safe=60;
	
			engines--;
		}
		if(safe>0)
		{
			safe--;
			danger=0;
		}
	}

	pinside=inside;
	
	cam_follow+=1.0f/240;
	if(cam_follow>1.0f)
		cam_follow=1.0f;

	float cfs=0.085f*pow(cam_follow, 2.0f);
	camera.forward+=(ship.forward-camera.forward)*cfs;
	camera.right+=(ship.right-camera.right)*cfs;
	camera.up+=(ship.up-camera.up)*cfs;
	camera.pos+=(ship.pos-camera.pos)*cfs*0.1f;
	RunPork(camera);
	Orthon(camera);

	for(int j=0;j<splodec;j++)
		if(splode[j].t>0)
		{
			splode[j].t--;
			splode[j].pos+=splode[j].vel;
			splode[j].vel*=0.95f;
		}
	
	if(stage==3)
	{
		static int wt=0;
		wt++;
		float f=(float)wt/150;
		Vector wpos=Vector(cos(f)+sin(f*1.09f), sin(f*1.7f)-cos(f*0.76f), sin(f*0.47f)-cos(f*1.3f))*1.0f;
		wpos*=sin(f*1.43f);
		float wd=wpos.Length();
		wd=spow(wd/3.0f, 0.5f)*3.0f;
		wpos=Normalize(wpos)*wd;
		
		wurmic++;
		wurmic%=wurmc;
		wurmf0+=0.67f;
		wurm[wurmic]=wpos;
		wurmf[wurmic]=wurmf0;
	}
	
	return true;
}

